#!/usr/bin/perl
#
# the script is modification of thermo_RD.pl written by Assen Roguev, 2006
# questions on the code to: roguev@cmp.ucsf.edu or stoynov@obzor.bio21.bas.bg
#
# thermo_RD.pl
# usage:
#		./thermo_NN.pl input_file start_position stop_position 2 1 > output_file 
# 
# indicate nearest-neighbor interactions across the DNA-RNA duplexes where RNA is equivalent to the Watson coding strand, using a sliding window with size 2 and step 1
# NN interactions are indicated by digits from 1 to 16 as follows:
# rAA/dTT=1
# rUU/dAA=2
# rAU/dTA=3
# rUA/dAT=4
# rCA/dGT=5
# rUG/dAC=6
# rGU/dCA=7
# rAC/dTG=8
# rCU/dGA=9
# rAG/dTC=10
# rGA/dCT=11
# rUC/dAG=12
# rCG/dGC=13 
# rGC/dCG=14
# rGG/dCC=15
# rCC/dGG=16
# 
# this software is freeware
# feel free to distribute and modify it
#

# let the code begin
use strict;

# hash of array  
# hash keys are respective pairs
# array element is digits corresponding to particular nearest-neighbor(NN) interaction see above

my %data_hash = (
'AA' => [1],
'TT' => [2],
'AT' => [3],
'TA' => [4],
'CA' => [5],
'TG' => [6],
'GT' => [7],
'AC' => [8],
'CT' => [9],
'AG' => [10],
'GA' => [11],
'TC' => [12],
'CG' => [13],
'GC' => [14],
'GG' => [15],
'CC' => [16]
);

# input parameters for the script
my $seq_file = shift;			# sequence file
my $start = shift;				# calculation start position 
my $stop = shift;				# calculation end position
my $window = shift;				# window size
my $step = shift;				# step

# some variables we are gonna use later
# define them here as globals to avoid scope issues

my $G;				# digits indicating NN interactions
my $roundG;			# rounded G, used for the output
my $input_seq;		# input sequence after stiching the lines	
my $current_pos;	# current position within the sequence
my $seq;			# working sequence
my $N_character = 'N';

# open the input file
open(IN,$seq_file) || die "Cannot open $seq_file\n";

# read line by line
while (my $line = <IN>)					# read a line
	{
	chomp($line);						# remove new line characters at the end of the lines
	$input_seq = $input_seq.$line;		# stitch the line with the rest
	}
close(IN);								# close input file

# define the working sequence based on start and stop position 
$seq = substr($input_seq, $start, ($stop-$start)); 

# do the maths
for (my $i = 0; $i <= length($seq) - $window; $i+= $step)	# incerement by the given step
	{
	# zero the variables
	$G = 0;

	# define a subsequnce based on the window size 
	my $sub_seq = substr($seq,$i,$window);

	# removes sequences with Ns in them
	if ($sub_seq =~ /$N_character/) { next; }

	# extract all the nearest neighbour pairs
	for (my $j = 0; $j < length($sub_seq); $j++)
		{
		my $pair = substr($sub_seq,$j,2);
	
		$pair = uc($pair);		# convert to upper case
		
		# get respective values from the hash		
		$G += $data_hash{$pair}->[0];
		}
		
	$roundG = sprintf "%.2f",$G;	# round the value	
	$current_pos = $start + 1 + $i + $window/2;		# calculate where we are now
	print "$current_pos\t$roundG\n";				# print output on screen
	}
# output file is txt delimited file with two columns: the first one indicates the position of the second nucleotide in the nearest-neighbor(NN) couple; the second represents digits corresponding to particular NN interaction.